using System.IO;
using UnityEditor;
using UnityEngine;
using UnityEngine.Rendering;

namespace Unity.XR.HisenseXR.Editor
{
    [InitializeOnLoad]
    public class HisXRProjectSettings : EditorWindow
    {
        public static HisXRProjectSettings window;
        public static string assetPath = "Assets/XR/Settings/";
        GUIContent myTitleContent = new GUIContent();
        static ELanguage eLanguage = ELanguage.Chinese;

        const BuildTarget recommended_BuildTarget = BuildTarget.Android;
        const int recommended_vSyncCount = 0;
        const UIOrientation recommended_orientation = UIOrientation.LandscapeLeft;

        const AndroidSdkVersions min_sdk_versions = AndroidSdkVersions.AndroidApiLevel29;
        const AndroidSdkVersions target_sdk_versions = AndroidSdkVersions.AndroidApiLevelAuto;

        const bool multithreaded_rendering = false;

        const GraphicsDeviceType graphicsDeviceType = GraphicsDeviceType.OpenGLES3;
        const bool auto_Graphics_api = false;

        const AndroidArchitecture targetArchitecture = AndroidArchitecture.ARM64;

        public bool toggleBuildTarget = true;
        public bool toggleVSync = true;
        public bool toggleOrientation = true;
        public bool toggleMinSDKVersion = true;
        public bool toggleTargetSDKVersion = true;
        public bool toggleMultithreadedRendering = true;
        public bool toggleOpenGLES3GraphicsAPI = true;
        public bool toggleSafeZoneLayer = true;
        public bool toggleSafeZoneSortingLayer = true;
        public bool toggleAndroidArchitecture = true;




        static string mCurrentAPI = "";

        static bool mResetGraphicAPI = true;


        GUIStyle styleApply;

        #region strings
        static string[] strWindowName = { "HisenseXRPlugin Project Setting", "HisenseXRPlugin Project 设置" };
        string strseparate = "______________________________________________________________________________________________________________________________________________";
        string[] strNoticeText = { "Notice: Recommended project settings for HisenseXRPlugin", "注意：HisenseXRPlugin 推荐项目配置" };
        string[] strBtnChange = { "切换至中文", "Switch to English" };
        string[] strApplied = { "Applied", "已应用" };

        string[] strInformationText = { "Information:", "信息说明" };

        string[] strInfo1Text =
        {
        "1 Support Unity Version: Unity 2020.3.0 Unity 2020.3.21 Unity 2020.3.40 Unity 2020.3.47 Unity 2021.3.0 Unity 2021.3.16 Unity 2021.3.23 Unity 2022.2.17",
        "1 支持Unity版本：Unity 2020.3.0 Unity 2020.3.21 Unity 2020.3.40 Unity 2020.3.47 Unity 2021.3.0 Unity 2021.3.16 Unity 2021.3.23 Unity 2022.2.17"
    };

        string[] strInfo2Text =
        {
        "2 The official website:",
        "2 官网:"
    };

        string[] strInfoURL = { "https://web-hicloudvr.hismarttv.com/web/#/microhome?id=1112", "https://web-hicloudvr.hismarttv.com/web/#/microhome?id=1112" };
        string[] strConfigurationText = { "Configuration:", "配置" };
        string[] strConfiguration1Text =
        {
        "1 Current:             Build Target = {0}\n" +
        "   Recommended:  Build Target = {1}\n",
        "1 当前:             Build Target = {0}\n" +
        "   推荐:             Build Target = {1}\n"
    };

        string[] strConfiguration2Text =
        {
        "2 Current:             V Sync Count = {0}\n" +
        "   Recommended:  V Sync Count = {1}\n",
        "2 当前:             V Sync Count = {0}\n" +
        "   推荐:             V Sync Count = {1}\n"
    };

        string[] strConfiguration3Text =
        {
        "3 Current:             Orientation = {0}\n" +
        "   Recommended:  Orientation = {1}\n",
        "3 当前:             Orientation = {0}\n" +
        "   推荐:             Orientation = {1}\n"
    };


        string[] strConfiguration4Text =
        {
        "4 Current:             Minimum API Level = {0}\n" +
        "   Recommended:  Minimum API Level = {1}\n",
        "4 当前:             Minimum API Level = {0}\n" +
        "   推荐:             Minimum API Level = {1}\n"
    };

        string[] strConfiguration5Text =
        {
        "5 Current:             Target API Level = {0}\n" +
        "   Recommended:  Target API Level = {1}\n",
        "5 当前:             Target API Level = {0}\n" +
        "   推荐:             Target API Level = {1}\n"
    };

        string[] strConfiguration6Text =
        {
        "6 Current:             Graphics API = {0}\n" +
        "   Recommended:   Graphics API = {1}\n",
        "6 当前:              Graphics API = {0}\n" +
        "   推荐:              Graphics API = {1}\n"
    };

        string[] strConfiguration7Text =
    {
        "7 Current:             Multithreaded Rendering = {0}\n" +
        "   Recommended:   Multithreaded Rendering = {1}\n",
        "7 当前:              Multithreaded Rendering = {0}\n" +
        "   推荐:              Multithreaded Rendering = {1}\n"
    };
        string[] strConfiguration8Text =
    {
        "2 Current:             Target Architectures = {0}\n" +
        "   Recommended:   Target Architectures = {1}\n",
        "2 当前:              Target Architectures = {0}\n" +
        "   推荐:              Target Architectures = {1}\n"
    };


        //     string[] strConfiguration8Text =
        // {
        //         "8 Current:             SafetyZone layers Active= {0}\n" +
        //         "   Recommended:   SafetyZone layers Active= {1}\n",
        //         "8 当前:              SafetyZone layers  Active= {0}\n" +
        //         "   推荐:              SafetyZone layers Active = {1}\n"
        //     };

        //     string[] strConfiguration9Text =
        // {
        //         "9 Current:             SafetyZone SortingLayers Active = {0}\n" +
        //         "   Recommended:   SafetyZone SortingLayers Active = {1}\n",
        //         "9 当前:              SafetyZone SortingLayers Active = {0}\n" +
        //         "   推荐:              SafetyZone SortingLayers Active = {1}\n"
        //     };

        string[] strBtnApply = { "Apply", "应用" };
        string[] strBtnClose = { "Close", "关闭" };

        #endregion


        static HisXRProjectSettings()
        {
            EditorApplication.update += Update;
        }

        [MenuItem("HisenseXR" + "/HisenseXRSetting")]
        public static void Init()
        {
            IsIgnoreWindow();
            ShowSettingWindow();
        }

        static void Update()
        {
            if (EditorApplication.isCompiling) return;

            bool allApplied = IsAllApplied();
            bool showWindow = !allApplied;
            if (IsIgnoreWindow())
            {
                showWindow = false;
            }
            if (showWindow)
            {
                ShowSettingWindow();
            }

            EditorApplication.update -= Update;
        }

        public static bool IsIgnoreWindow()
        {
            string path = HisXRProjectSettings.assetPath + typeof(HisXRProjectSettingAsset).ToString() + ".asset";
            if (File.Exists(path))
            {
                HisXRProjectSettingAsset asset = AssetDatabase.LoadAssetAtPath<HisXRProjectSettingAsset>(path);
                return asset.IgnoreSDKSetting;
            }
            return false;
        }

        static void ShowSettingWindow()
        {
            if (window != null)
                return;
            window = (HisXRProjectSettings)GetWindow(typeof(HisXRProjectSettings), true, strWindowName[(int)eLanguage], true);
            window.autoRepaintOnSceneChange = true;
            window.minSize = new Vector2(960, 620);
        }

        string GetResourcePath()
        {
            var ms = MonoScript.FromScriptableObject(this);
            var path = AssetDatabase.GetAssetPath(ms);
            path = Path.GetDirectoryName(path);
            return path.Substring(0, path.Length - "Editor".Length) + "Textures/";
        }

        public void OnGUI()
        {
            myTitleContent.text = strWindowName[(int)eLanguage];

            if (window != null)
            {
                window.titleContent = myTitleContent;
            }
            ShowNoticeTextandChangeBtn();

            GUIStyle styleSlide = new GUIStyle();
            styleSlide.normal.textColor = Color.white;
            GUILayout.Label(strseparate, styleSlide);

            GUILayout.Label(strInformationText[(int)eLanguage]);

            GUIStyle strInfo1TextStyle = new GUIStyle();
            strInfo1TextStyle.normal.textColor = Color.gray;
            GUILayout.Label(strInfo1Text[(int)eLanguage], strInfo1TextStyle);
            GUILayout.Label(strInfo2Text[(int)eLanguage], strInfo1TextStyle);
            string strURL = strInfoURL[(int)eLanguage];
            GUIStyle style = new GUIStyle();
            style.normal.textColor = new Color(0, 122f / 255f, 204f / 255f);
            if (GUILayout.Button("   " + strURL, style, GUILayout.Width(400)))
            {
                Application.OpenURL(strURL);
            }

            GUILayout.Label(strseparate, styleSlide);


            GUILayout.Label(strConfigurationText[(int)eLanguage]);


            string strInfo1 = string.Format(strConfiguration1Text[(int)eLanguage], EditorUserBuildSettings.activeBuildTarget, recommended_BuildTarget);
            EditorConfigurations(strInfo1, EditorUserBuildSettings.activeBuildTarget == recommended_BuildTarget, ref toggleBuildTarget);

            // string strInfo2 = string.Format(strConfiguration2Text[(int)eLanguage], QualitySettings.vSyncCount,
            //     recommended_vSyncCount);
            // EditorConfigurations(strInfo2, QualitySettings.vSyncCount == recommended_vSyncCount, ref toggleVSync);

            string strInfo2 = string.Format(strConfiguration8Text[(int)eLanguage], PlayerSettings.Android.targetArchitectures,
                targetArchitecture);
            EditorConfigurations(strInfo2, PlayerSettings.Android.targetArchitectures == targetArchitecture, ref toggleAndroidArchitecture);

            string strInfo3 = string.Format(strConfiguration3Text[(int)eLanguage],
                PlayerSettings.defaultInterfaceOrientation, recommended_orientation);
            EditorConfigurations(strInfo3, PlayerSettings.defaultInterfaceOrientation == recommended_orientation,
                ref toggleOrientation);

            string strInfo4 = string.Format(strConfiguration4Text[(int)eLanguage],
            PlayerSettings.Android.minSdkVersion, min_sdk_versions);
            EditorConfigurations(strInfo4, PlayerSettings.Android.minSdkVersion == min_sdk_versions,
                ref toggleMinSDKVersion);

            string strInfo5 = string.Format(strConfiguration5Text[(int)eLanguage],
            PlayerSettings.Android.targetSdkVersion, target_sdk_versions);
            EditorConfigurations(strInfo5, PlayerSettings.Android.targetSdkVersion == target_sdk_versions,
                ref toggleTargetSDKVersion);

            if (string.IsNullOrEmpty(mCurrentAPI))
            {
                GetCurrentGraphicsAPIs();
            }
            string strInfo6 = string.Format(strConfiguration6Text[(int)eLanguage], mCurrentAPI.Substring(0, mCurrentAPI.Length - 1), graphicsDeviceType);
            EditorConfigurations(strInfo6, mResetGraphicAPI == auto_Graphics_api,
                ref toggleOpenGLES3GraphicsAPI);


            // string strInfo7 = string.Format(strConfiguration7Text[(int)eLanguage],
            //  PlayerSettings.GetMobileMTRendering(BuildTargetGroup.Android), multithreaded_rendering);
            // EditorConfigurations(strInfo7, PlayerSettings.GetMobileMTRendering(BuildTargetGroup.Android) == multithreaded_rendering,
            //     ref toggleMultithreadedRendering);

            EditorGUILayout.Space();
            EditorGUILayout.BeginHorizontal();
            EditorGUILayout.LabelField("", GUILayout.Width(200));

            styleApply = new GUIStyle("LargeButton");
            styleApply.alignment = TextAnchor.MiddleCenter;

            if (GUILayout.Button(strBtnApply[(int)eLanguage], styleApply, GUILayout.Width(100), GUILayout.Height(30)))
            {
                EditorApplication.delayCall += OnClickApply;
            }
            styleApply = null;

            EditorGUILayout.LabelField("", GUILayout.Width(200));
            if (GUILayout.Button(strBtnClose[(int)eLanguage], GUILayout.Width(100), GUILayout.Height(30)))
            {
                OnClickClose();
            }
            EditorGUILayout.EndHorizontal();
        }

        private static bool GetCurrentGraphicsAPIs()
        {
            GraphicsDeviceType[] currentDeviceType = PlayerSettings.GetGraphicsAPIs(BuildTarget.Android);
            if (currentDeviceType != null)
            {
                mCurrentAPI = "";
                for (int i = 0; i < currentDeviceType.Length; i++)
                {
                    mCurrentAPI = mCurrentAPI + currentDeviceType[i].ToString() + "/";
                }
            }

            if (mCurrentAPI.Equals(GraphicsDeviceType.OpenGLES3 + "/"))
            {
                mResetGraphicAPI = false;
                mCurrentAPI = GraphicsDeviceType.OpenGLES3.ToString() + "/";
                return true;
            }
            else
            {
                mResetGraphicAPI = true;
                return false;
            }
        }

        public void OnClickApply()
        {

            if (toggleVSync && QualitySettings.vSyncCount != recommended_vSyncCount)
            {
                QualitySettings.vSyncCount = recommended_vSyncCount;
            }


            if (toggleOrientation && PlayerSettings.defaultInterfaceOrientation != recommended_orientation)
            {
                PlayerSettings.defaultInterfaceOrientation = recommended_orientation;
            }

            if (toggleBuildTarget && EditorUserBuildSettings.activeBuildTarget != recommended_BuildTarget)
            {
                EditorUserBuildSettings.SwitchActiveBuildTarget(BuildTargetGroup.Android, recommended_BuildTarget);
                EditorUserBuildSettings.selectedBuildTargetGroup = BuildTargetGroup.Android;
            }
            if (toggleMinSDKVersion && PlayerSettings.Android.minSdkVersion != min_sdk_versions)
            {
                PlayerSettings.Android.minSdkVersion = AndroidSdkVersions.AndroidApiLevel29;
            }
            if (toggleTargetSDKVersion && PlayerSettings.Android.targetSdkVersion != target_sdk_versions)
            {
                PlayerSettings.Android.targetSdkVersion = AndroidSdkVersions.AndroidApiLevelAuto;
            }

            if (toggleMultithreadedRendering && PlayerSettings.GetMobileMTRendering(BuildTargetGroup.Android) != multithreaded_rendering)
            {
                PlayerSettings.SetMobileMTRendering(BuildTargetGroup.Android, multithreaded_rendering);
            }

            if (toggleOpenGLES3GraphicsAPI && mResetGraphicAPI != auto_Graphics_api)
            {
                PlayerSettings.SetUseDefaultGraphicsAPIs(BuildTarget.Android, auto_Graphics_api);
                PlayerSettings.SetGraphicsAPIs(BuildTarget.Android, new GraphicsDeviceType[] { GraphicsDeviceType.OpenGLES3 });
                GetCurrentGraphicsAPIs();
            }

            if (toggleAndroidArchitecture && PlayerSettings.Android.targetArchitectures != targetArchitecture)
            {
                PlayerSettings.Android.targetArchitectures = AndroidArchitecture.ARM64;
            }
        }

        void OnClickClose()
        {
            bool allApplied = IsAllApplied();
            if (allApplied)
            {
                Close();
            }
            else
            {
                HisSettingMessageBoxEditor.Init(eLanguage);
            }
        }

        public static bool IsAllApplied()
        {
            return (EditorUserBuildSettings.activeBuildTarget == recommended_BuildTarget) &&
                            (PlayerSettings.Android.targetArchitectures == targetArchitecture) &&
                            (PlayerSettings.defaultInterfaceOrientation == recommended_orientation) && (PlayerSettings.Android.minSdkVersion == min_sdk_versions) && (PlayerSettings.Android.targetSdkVersion == target_sdk_versions) && GetCurrentGraphicsAPIs();
        }

        void EditorConfigurations(string strConfiguration, bool torf, ref bool toggle)
        {
            EditorGUILayout.BeginHorizontal();

            GUILayout.Label(strConfiguration, GUILayout.Width(500));

            GUIStyle styleApplied = new GUIStyle();
            styleApplied.normal.textColor = Color.green;
            if (torf)
            {
                GUILayout.Label(strApplied[(int)eLanguage], styleApplied);
            }
            else
            {
                toggle = EditorGUILayout.Toggle(toggle);
            }

            EditorGUILayout.EndHorizontal();
        }

        void ShowLogo()
        {
            var resourcePath = GetResourcePath();
#if !(UNITY_5_0)
            var logo = AssetDatabase.LoadAssetAtPath<Texture2D>(resourcePath + "logo.png");
#else
		var logo = Resources.LoadAssetAtPath<Texture2D>(resourcePath + "logo.png");
#endif
            if (logo)
            {
                var rect = GUILayoutUtility.GetRect(position.width, 150, GUI.skin.box);
                GUI.DrawTexture(rect, logo, ScaleMode.ScaleToFit);
            }
        }

        void ShowNoticeTextandChangeBtn()
        {
            EditorGUILayout.BeginHorizontal();

            GUIStyle styleNoticeText = new GUIStyle();
            styleNoticeText.alignment = TextAnchor.MiddleCenter;
            styleNoticeText.fontSize = 22;
            styleNoticeText.normal.textColor = Color.white;
            GUILayout.Label(strNoticeText[(int)eLanguage], styleNoticeText);

            if (GUILayout.Button(strBtnChange[(int)eLanguage], GUILayout.Width(150), GUILayout.Height(25)))
            {
                SwitchLanguage();
            }

            EditorGUILayout.EndHorizontal();
        }

        void SwitchLanguage()
        {
            if (eLanguage == ELanguage.Chinese)
                eLanguage = ELanguage.English;
            else if (eLanguage == ELanguage.English)
                eLanguage = ELanguage.Chinese;
        }

        void OnDestroy()
        {
        }
    }

    public enum ELanguage
    {
        English,
        Chinese,
    }

    public class HisSettingMessageBoxEditor : EditorWindow
    {
        static HisSettingMessageBoxEditor myWindow;
        static ELanguage elanguage = ELanguage.English;
        static string[] strWindowName = { "Ignore the recommended configuration", "忽略推荐配置" };
        string[] strTipInfo = { "                                   No more prompted \n" +
            "             You can get recommended configuration from  \n" +
            "                            Development documentation.",
             "                               点击\"忽略\"后,不再提示！\n"+
            "                       可从开发者文档中获取推荐配置说明  \n"};

        string[] strBtnIgnore = { "Ignore", "忽略" };
        string[] strBtnCancel = { "Cancel", "取消" };

        public static void Init(ELanguage language)
        {
            elanguage = language;
            myWindow = (HisSettingMessageBoxEditor)GetWindow(typeof(HisSettingMessageBoxEditor), true, strWindowName[(int)language], true);
            myWindow.autoRepaintOnSceneChange = true;
            myWindow.minSize = new Vector2(360, 200);
            myWindow.Show(true);
            Rect pos;
            if (HisXRProjectSettings.window != null)
            {
                Rect frect = HisXRProjectSettings.window.position;
                pos = new Rect(frect.x + 300, frect.y + 200, 200, 140);
            }
            else
            {
                pos = new Rect(700, 400, 200, 140);
            }
            myWindow.position = pos;
        }

        void OnGUI()
        {
            for (int i = 0; i < 10; i++)
            {
                EditorGUILayout.Space();
            }
            GUILayout.Label(strTipInfo[(int)elanguage]);


            for (int i = 0; i < 3; i++)
            {
                EditorGUILayout.Space();
            }
            EditorGUILayout.BeginHorizontal();
            EditorGUILayout.LabelField("", GUILayout.Width(20));
            if (GUILayout.Button(strBtnIgnore[(int)elanguage], GUILayout.Width(100), GUILayout.Height(30)))
            {
                OnClickIgnore();
            }
            EditorGUILayout.LabelField("", GUILayout.Width(50));
            if (GUILayout.Button(strBtnCancel[(int)elanguage], GUILayout.Width(130), GUILayout.Height(30)))
            {
                OnClickCancel();
            }
            EditorGUILayout.EndHorizontal();
        }

        void OnClickIgnore()
        {
            SaveAssetDataBase();
            HisXRProjectSettings.window.Close();
            Close();
        }

        private void SaveAssetDataBase()
        {
            HisXRProjectSettingAsset asset;
            string assetPath = HisXRProjectSettings.assetPath + typeof(HisXRProjectSettingAsset).ToString() + ".asset";
            if (File.Exists(assetPath))
            {
                asset = AssetDatabase.LoadAssetAtPath<HisXRProjectSettingAsset>(assetPath);
            }
            else
            {
                asset = ScriptableObject.CreateInstance<HisXRProjectSettingAsset>();
                ScriptableObjectUtility.CreateAsset<HisXRProjectSettingAsset>(asset, HisXRProjectSettings.assetPath);
            }
            asset.IgnoreSDKSetting = true;
            EditorUtility.SetDirty(asset);
            AssetDatabase.SaveAssets();
            AssetDatabase.Refresh();//must Refresh
        }
        void OnClickCancel()
        {
            Close();
        }

        public static class ScriptableObjectUtility
        {
            public static void CreateAsset<T>(T classdata, string path) where T : ScriptableObject
            {
                if (!Directory.Exists(path))
                {
                    Directory.CreateDirectory(path);
                }
                string assetPathAndName = AssetDatabase.GenerateUniqueAssetPath(path + typeof(T).ToString() + ".asset");

                AssetDatabase.CreateAsset(classdata, assetPathAndName);

                AssetDatabase.SaveAssets();
                AssetDatabase.Refresh();
            }
        }
    }
}