﻿using System.Collections.Generic;
using UnityEditor;

#if UNITY_EDITOR
using UnityEditor.XR.OpenXR.Features;
using UnityEngine.XR.OpenXR.Features.Interactions;
#endif

namespace UnityEngine.XR.OpenXR.Features.HisSupport
{
#if UNITY_EDITOR
    [OpenXRFeature(UiName = "Hisense XR Support",
        Desc = "Hisense xr loader feature.",
        Company = "",
        Version = "1.0.1",
        BuildTargetGroups = new[] { BuildTargetGroup.Android },
        CustomRuntimeLoaderBuildTargets = new[] { BuildTarget.Android },
        FeatureId = featureId
    )]
#endif
    public class HisOpenXRLoaderFeature : OpenXRFeature
    {
        /// <summary>
        /// The feature id string. This is used to give the feature a well known id for reference.
        /// </summary>
        public const string featureId = "com.unity.openxr.feature.his";

#if UNITY_EDITOR
        protected override void GetValidationChecks(List<ValidationRule> rules, BuildTargetGroup targetGroup)
        {
            rules.Add(new ValidationRule(this)
            {
                message = "Only the Touch Interaction Profile is supported right now.",
                checkPredicate = () =>
                {
                    var settings = OpenXRSettings.GetSettingsForBuildTargetGroup(targetGroup);
                    if (null == settings)
                        return false;

                    return true;
                },
                fixIt = () =>
                {
                    var settings = OpenXRSettings.GetSettingsForBuildTargetGroup(targetGroup);
                    if (null == settings)
                        return;
                },
                error = true,
            });
        }
#endif

        public static void CheckEnvironmentBlendMode(UnityEngine.XR.OpenXR.NativeTypes.XrEnvironmentBlendMode mode)
        {
            //if (GetEnvironmentBlendMode() != mode)
            //{
            //    Debug.Log($"Environment blend mode {mode}");

                SetEnvironmentBlendMode(mode);
            //}
        }
    }
}
