using UnityEngine.XR.Interaction.Toolkit;

namespace UnityEngine.XR.OpenXR.Features.HisSupport
{
    public class HisControllerSwitcher : MonoBehaviour
    {
        [Header("Head Gaze Mode")]
        public ActionBasedController GazeController;

        [Header("Left Hand Controller")]
        public ActionBasedController LeftHandController;

        [Header("Right Hand Controller")]
        public ActionBasedController RightHandController;

        public HisHand leftHand;
        public HisHand rightHand;


        InputDevice LeftHandDevice;
        InputDevice RightHandDevice;

        private void Start()
        {
            LeftHandDevice = InputDevices.GetDeviceAtXRNode(XRNode.LeftHand);
            RightHandDevice = InputDevices.GetDeviceAtXRNode(XRNode.RightHand);
            ControllerLeftController(false);
            ControllerRightController(false);
            ControllerGazeMode(false);

            Invoke("StartHandTrack", 2);
        }


        public void StartHandTrack()
        {
            Debug.Log("HisControllerSwitcher:StartHandTrack");
            HisHandManager.Instance.isHandOn = true;
            HisHandManager.Instance.StartHandTrack();
        }

        private void Update()
        {
            LeftHandDevice.TryGetFeatureValue(CommonUsages.isTracked, out bool leftHandState);
            RightHandDevice.TryGetFeatureValue(CommonUsages.isTracked, out bool rightHandState);

            if (leftHandState || rightHandState)
            {
                ControllerGazeMode(false);

                ControllerLeftController(leftHandState);
                ControllerRightController(rightHandState);
                leftHand.SetRayDisplayState = false;
                rightHand.SetRayDisplayState = false;
            }
            else
            {
                ControllerGazeMode(true);
                ControllerLeftController(false);
                ControllerRightController(false);
                leftHand.SetRayDisplayState = false;
                rightHand.SetRayDisplayState = false;
            }
        }

        void ControllerGazeMode(bool enabled)
        {
            GazeController.hideControllerModel = !enabled;
            GazeController.enabled = enabled;
            GazeController.gameObject.SetActive(enabled);
        }

        void ControllerLeftController(bool enabled)
        {
            LeftHandController.hideControllerModel = !enabled;
            LeftHandController.gameObject.SetActive(enabled);
        }

        void ControllerRightController(bool enabled)
        {
            RightHandController.hideControllerModel = !enabled;
            RightHandController.gameObject.SetActive(enabled);
        }
    }
}

