Shader "Custom/Hand"
{
    Properties
    {
        [Header(BaseColor)]
        _ColorTop("ColorTop",Color) = (1,1,1,1)
        _ColorBottom("ColorBottom",Color) = (1,1,1,1)
        _FresnelPower("Fresnel",float) = 1

        [Header(OutLine)][Space(5)]
        _OutColor("OutColor",Color) = (1,1,1,1)
        _OutlineWidth("outline width",Range(0.01,1)) = 0.05
        _OutlineOpacity("Outline Opacity", Range(0 , 1)) = 0.4

        [Header(Light)][Space(5)]
        _ClickLight("ClickLight",Color) = (1,1,1,1)
        _ClickRange("ClickRange",Range(0,1)) = 0.015
        _ClickPosition("ClickPosition",Vector) = (1,1,1,1)
    }

    CGINCLUDE
    #include "Lighting.cginc"
    #pragma target 3.0

    float4 _ColorTop;
    float4 _ColorBottom;
    float4 _OutColor;
    float _FresnelPower;

    float4 _ClickLight;
    float4 _ClickPosition;
    float _ClickRange;

    float _OutlineWidth;
    float _OutlineOpacity;
    ENDCG

    SubShader
    {
        LOD 200

        Tags
        {
            "Queue" = "Transparent" "RenderType" = "Transparent" "IgnoreProjector" = "True"
        }
        
        Pass
        {
            ZWrite On
            ColorMask 0
        }

        Pass
        {
            Tags
            {
                "RenderType" = "Transparent" "Queue" = "Transparent" "IgnoreProjector" = "True"
            }
            Cull Front
            Blend SrcAlpha OneMinusSrcAlpha

            CGPROGRAM
            #pragma vertex vert
            #pragma fragment frag
            struct OutlineInput
            {
                float4 vertex : POSITION;
                float3 normal : NORMAL;
                //float4 color : COLOR;
            };

            struct OutlineOutput
            {
                float4 vertex : SV_POSITION;
                //float4 color : COLOR;
            };

            OutlineOutput vert(OutlineInput v)
            {
                OutlineOutput o;
                v.vertex.xyz += v.normal * _OutlineWidth * 0.01;
                o.vertex = UnityObjectToClipPos(v.vertex);
                //o.color = v.color;
                return o;
            }

            fixed4 frag(OutlineOutput v) : SV_Target
            {
                half4 color = half4(0, 0, 0, 1);
                color.rgb = _OutColor.rgb;//lerp(v.color, _OutColor, 0.1);
                color.a = _OutColor.a * _OutlineOpacity;
                return color;
            }
            ENDCG
        }

        Pass
        {
            Tags
            {
                "RenderType" = "MaskedOutline" "Queue" = "Transparent" "IgnoreProjector" = "True" "IsEmissive" = "true"
            }
            Blend SrcAlpha OneMinusSrcAlpha

            CGPROGRAM
            #pragma vertex vert
            #pragma fragment frag
            struct VertexInput
            {
                float4 vertex : POSITION;
                half3 normal : NORMAL;
                half4 vertexColor : COLOR;
                float2 texcoord : TEXCOORD0;
            };

            struct VertexOutput
            {
                float4 vertex : SV_POSITION;
                float3 worldPos : TEXCOORD1;
                float3 worldNormal : TEXCOORD2;
                float2 uv:TEXCOORD3;
            };

            void Remap(in float4 inValue, float2 inMinMax, float2 outMinMax, out float4 outValue)
            {
                outValue = outMinMax.x + (inValue - inMinMax.x) * (outMinMax.y - outMinMax.x) / (inMinMax.y - inMinMax.x);
            }

            VertexOutput vert(VertexInput v)
            {
                VertexOutput o;
                o.worldPos = mul(unity_ObjectToWorld, v.vertex);
                o.worldNormal = UnityObjectToWorldNormal(v.normal);
                o.vertex = UnityObjectToClipPos(v.vertex);
                o.uv = v.texcoord;
                return o;
            }

            fixed4 frag(VertexOutput v) : SV_Target
            {
                float3 normalWS = normalize(v.worldNormal);
                float3 viewWS = normalize(UnityWorldSpaceViewDir(v.worldPos));
                float fresnel = pow(1 -saturate(dot(viewWS, normalWS)), _FresnelPower);
                float4 baseColor = lerp(_ColorTop, _ColorBottom, fresnel);

                float3 localClickPos = mul((float3x3)unity_WorldToObject, _ClickPosition);
                float3 vertexPos = mul((float3x3)unity_WorldToObject, v.worldPos);
                float dis = distance(localClickPos, vertexPos);

                float2 inMinMax = float2(0, _ClickRange);
                float2 outMinMax = float2(1, 0);
                float4 s;
                Remap(dis, inMinMax, outMinMax, s);
                float r = smoothstep(0, 1, clamp(s, 0, 1));

                fixed4 finalCol = lerp(baseColor, _ClickLight, r);
                return finalCol;
            }
            ENDCG
        }
    }
}