using UnityEngine;
using UnityEngine.XR.Interaction.Toolkit;
using static UnityEngine.XR.OpenXR.Features.HisSupport.HisHandDll;

namespace UnityEngine.XR.OpenXR.Features.HisSupport
{
    public class HisHand : MonoBehaviour
    {

        public HandType handType;
        public Transform[] handJoints;
        private Transform mRayPose;
        private GameObject mDefaultRay;
        private SkinnedMeshRenderer[] touchRenders;
        public bool Computed { get; private set; }
        public Posef RayPose { get; private set; }
        public bool Pinch { get; private set; }
        public float PinchStrength { get; private set; }
        public bool RayValid { get; private set; }
        public bool RayTouched { get; private set; }
        public float TouchStrengthRay { get; private set; }
        public bool IndexPinching { get; private set; }
        public bool MiddlePinching { get; private set; }
        public bool RingPinching { get; private set; }
        public bool LittlePinching { get; private set; }
        public float PinchStrengthIndex { get; private set; }
        public float PinchStrengthMiddle { get; private set; }
        public float PinchStrengthRing { get; private set; }
        public float PinchStrengthLittle { get; private set; }

        private Transform mRayOrigin;

        private HandAimState aimState = new HandAimState();
        private Transform mHead;

        private bool mIsDisplayRayState = true;

        public Vector3 IndexTip
        {
            get { return handJoints[9].position; }
        }

        /// <summary>
        /// 设置射线瞄点与指示器状态
        /// </summary>
        /// <param name="state"></param>
        public bool SetRayDisplayState
        {
            set
            {
                mIsDisplayRayState = value;
            }
        }


        void Awake()
        {
            handJoints = new Transform[22];
            mHead = Camera.main.transform;
            mRayPose = this.transform.Find("HisRayPose");
            mDefaultRay = mRayPose.Find("Hand_Ray").gameObject;
            mRayOrigin = mRayPose.Find("RayOrigin");
            if (mDefaultRay != null)
            {
                touchRenders = mDefaultRay.GetComponentsInChildren<SkinnedMeshRenderer>();
            }

            if (handType == HandType.Left)
            {
                handJoints[0] = this.transform.Find("b_l_wrist");

                handJoints[1] = this.transform.Find("b_l_wrist/b_l_thumb0");
                handJoints[2] = this.transform.Find("b_l_wrist/b_l_thumb0/b_l_thumb1");
                handJoints[3] = this.transform.Find("b_l_wrist/b_l_thumb0/b_l_thumb1/b_l_thumb2");
                handJoints[4] = this.transform.Find("b_l_wrist/b_l_thumb0/b_l_thumb1/b_l_thumb2/b_l_thumb3");
                handJoints[5] = this.transform.Find("b_l_wrist/b_l_thumb0/b_l_thumb1/b_l_thumb2/b_l_thumb3/l_thumb_finger_tip_marker");

                handJoints[6] = this.transform.Find("b_l_wrist/b_l_index1");
                handJoints[7] = this.transform.Find("b_l_wrist/b_l_index1/b_l_index2");
                handJoints[8] = this.transform.Find("b_l_wrist/b_l_index1/b_l_index2/b_l_index3");
                handJoints[9] = this.transform.Find("b_l_wrist/b_l_index1/b_l_index2/b_l_index3/l_index_finger_tip_marker");

                handJoints[10] = this.transform.Find("b_l_wrist/b_l_middle1");
                handJoints[11] = this.transform.Find("b_l_wrist/b_l_middle1/b_l_middle2");
                handJoints[12] = this.transform.Find("b_l_wrist/b_l_middle1/b_l_middle2/b_l_middle3");
                handJoints[13] = this.transform.Find("b_l_wrist/b_l_middle1/b_l_middle2/b_l_middle3/l_middle_finger_tip_marker");

                handJoints[14] = this.transform.Find("b_l_wrist/b_l_ring1");
                handJoints[15] = this.transform.Find("b_l_wrist/b_l_ring1/b_l_ring2");
                handJoints[16] = this.transform.Find("b_l_wrist/b_l_ring1/b_l_ring2/b_l_ring3");
                handJoints[17] = this.transform.Find("b_l_wrist/b_l_ring1/b_l_ring2/b_l_ring3/l_ring_finger_tip_marker");

                handJoints[18] = this.transform.Find("b_l_wrist/b_l_pinky0/b_l_pinky1");
                handJoints[19] = this.transform.Find("b_l_wrist/b_l_pinky0/b_l_pinky1/b_l_pinky2");
                handJoints[20] = this.transform.Find("b_l_wrist/b_l_pinky0/b_l_pinky1/b_l_pinky2/b_l_pinky3");
                handJoints[21] = this.transform.Find("b_l_wrist/b_l_pinky0/b_l_pinky1/b_l_pinky2/b_l_pinky3/l_pinky_finger_tip_marker");
            }
            else
            {

                handJoints[0] = this.transform.Find("b_r_wrist");

                handJoints[1] = this.transform.Find("b_r_wrist/b_r_thumb0");
                handJoints[2] = this.transform.Find("b_r_wrist/b_r_thumb0/b_r_thumb1");
                handJoints[3] = this.transform.Find("b_r_wrist/b_r_thumb0/b_r_thumb1/b_r_thumb2");
                handJoints[4] = this.transform.Find("b_r_wrist/b_r_thumb0/b_r_thumb1/b_r_thumb2/b_r_thumb3");
                handJoints[5] = this.transform.Find("b_r_wrist/b_r_thumb0/b_r_thumb1/b_r_thumb2/b_r_thumb3/r_thumb_finger_tip_marker");

                handJoints[6] = this.transform.Find("b_r_wrist/b_r_index1");
                handJoints[7] = this.transform.Find("b_r_wrist/b_r_index1/b_r_index2");
                handJoints[8] = this.transform.Find("b_r_wrist/b_r_index1/b_r_index2/b_r_index3");
                handJoints[9] = this.transform.Find("b_r_wrist/b_r_index1/b_r_index2/b_r_index3/r_index_finger_tip_marker");

                handJoints[10] = this.transform.Find("b_r_wrist/b_r_middle1");
                handJoints[11] = this.transform.Find("b_r_wrist/b_r_middle1/b_r_middle2");
                handJoints[12] = this.transform.Find("b_r_wrist/b_r_middle1/b_r_middle2/b_r_middle3");
                handJoints[13] = this.transform.Find("b_r_wrist/b_r_middle1/b_r_middle2/b_r_middle3/r_middle_finger_tip_marker");

                handJoints[14] = this.transform.Find("b_r_wrist/b_r_ring1");
                handJoints[15] = this.transform.Find("b_r_wrist/b_r_ring1/b_r_ring2");
                handJoints[16] = this.transform.Find("b_r_wrist/b_r_ring1/b_r_ring2/b_r_ring3");
                handJoints[17] = this.transform.Find("b_r_wrist/b_r_ring1/b_r_ring2/b_r_ring3/r_ring_finger_tip_marker");

                handJoints[18] = this.transform.Find("b_r_wrist/b_r_pinky0/b_r_pinky1");
                handJoints[19] = this.transform.Find("b_r_wrist/b_r_pinky0/b_r_pinky1/b_r_pinky2");
                handJoints[20] = this.transform.Find("b_r_wrist/b_r_pinky0/b_r_pinky1/b_r_pinky2/b_r_pinky3");
                handJoints[21] = this.transform.Find("b_r_wrist/b_r_pinky0/b_r_pinky1/b_r_pinky2/b_r_pinky3/r_pinky_finger_tip_marker");
            }
        }

        #region 手势驱动相关

        Vector3 vecTemp;

        public void UpdateHandScale(float scale)
        {
            vecTemp.Set(scale, scale, scale);
            handJoints[0].localScale = vecTemp;
        }

        public void UpdateHandInfo(HxrXrPosef[] pose)
        {
            //后续待决定是否修改，目前先不写循环
            handJoints[0].position = ChangeJointToVector3(pose[0].position);
#if UNITY_ANDROID && !UNITY_EDITOR
            handJoints[0].rotation = ChangeJointToRotation(pose[0].orientation);
            handJoints[1].localRotation =ChangeJointToRotation(pose[1].orientation);
            handJoints[2].localRotation =ChangeJointToRotation(pose[2].orientation);
            handJoints[3].localRotation =ChangeJointToRotation(pose[3].orientation);
            handJoints[4].localRotation =ChangeJointToRotation(pose[4].orientation);
            handJoints[5].localRotation =ChangeJointToRotation(pose[5].orientation);

            handJoints[6].localRotation =ChangeJointToRotation(pose[6].orientation);
            handJoints[7].localRotation =ChangeJointToRotation(pose[7].orientation);
            handJoints[8].localRotation =ChangeJointToRotation(pose[8].orientation);
            handJoints[9].localRotation =ChangeJointToRotation(pose[9].orientation);

            handJoints[10].localRotation =ChangeJointToRotation(pose[10].orientation);
            handJoints[11].localRotation =ChangeJointToRotation(pose[11].orientation);
            handJoints[12].localRotation =ChangeJointToRotation(pose[12].orientation);
            handJoints[13].localRotation =ChangeJointToRotation(pose[13].orientation);

            handJoints[14].localRotation =ChangeJointToRotation(pose[14].orientation);
            handJoints[15].localRotation =ChangeJointToRotation(pose[15].orientation);
            handJoints[16].localRotation =ChangeJointToRotation(pose[16].orientation);
            handJoints[17].localRotation =ChangeJointToRotation(pose[17].orientation);

            handJoints[18].localRotation =ChangeJointToRotation(pose[18].orientation);
            handJoints[19].localRotation =ChangeJointToRotation(pose[19].orientation);
            handJoints[20].localRotation =ChangeJointToRotation(pose[20].orientation);
            handJoints[21].localRotation =ChangeJointToRotation(pose[21].orientation);
#endif
        }

        private Vector3 ChangeJointToVector3(HxrXrVector3f jPoint)
        {
            Vector3 vecTemp = Vector3.zero;
            vecTemp.Set(jPoint.x, jPoint.y, jPoint.z);
            return vecTemp;
        }

        private Quaternion ChangeJointToRotation(HxrXrQuaternionf jPoint)
        {
            Quaternion quatTemp = Quaternion.identity;
            quatTemp.Set(jPoint.x, jPoint.y, jPoint.z, jPoint.w);
            return quatTemp;
        }

        private Vector3 GetRayDir()
        {
            return RayPose.pos - GetRayOrigin();
        }

        private Vector3 GetRayOrigin()
        {
            int variable = handType == HandType.Right ? 1 : -1;
            return mHead.position + variable * mHead.right * 0.1f - mHead.up * 0.14f;
        }

        public void UpdateAimState()
        {
            GetAimState(ref aimState);
            Computed = (aimState.aimStatus & HandAimStatus.AimComputed) != 0;
            RayPose = aimState.aimRayPose;
            RayValid = (aimState.aimStatus & HandAimStatus.AimRayValid) != 0;
            Pinch = (aimState.aimStatus & HandAimStatus.AimRayTouched) != 0;
            PinchStrength = aimState.touchStrengthRay;

        }

        private void GetAimState(ref HandAimState aimState)
        {
            aimState.aimStatus = HandAimStatus.AimRayValid;
            aimState.aimRayPose.dir = GetRayDir();
            aimState.aimRayPose.pos = handJoints[10].position;
            float distance = Vector3.Distance(handJoints[5].position, handJoints[9].position) * 10;
            aimState.touchStrengthRay = distance > 1 ? 0 : 1 - distance;
        }

        public void UpdateRayPose(bool valid = true)
        {
            if (mRayPose == null) return;

            if (RayValid && valid)
            {
                mRayPose.gameObject.SetActive(mIsDisplayRayState);
                mRayPose.position = RayPose.pos + RayPose.dir.normalized * 0.09f;
                mRayPose.forward = RayPose.dir;
                mRayOrigin.position = GetRayOrigin();
                mRayOrigin.forward = GetRayDir().normalized;
                if (mDefaultRay != null)
                {
                    foreach (var touchRender in touchRenders)
                    {
                        touchRender.SetBlendShapeWeight(0, aimState.touchStrengthRay * 100);
                    }
                }
            }
            else
            {
                mRayPose.gameObject.SetActive(false);
            }
        }

#if UNITY_ANDROID && !UNITY_EDITOR
        bool mIWave = false;
#endif
        public void HandTransformRecognition(HxrTransformRecognition trans_rec)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            mIWave = trans_rec == HxrTransformRecognition.HXR_TRANSFORM_WRIST_DOWN;
#endif
        }
        #endregion

        #region 对外提供的接口

        public Vector3 GetHandRayDir()
        {
            return GetRayDir();
        }

        public Vector3 GetHandRayOrigin()
        {
            return mRayPose.transform.position;
        }
        #endregion
    }
}
