﻿using System;
using System.Collections;
using System.Runtime.InteropServices;
using UnityEngine;
using static UnityEngine.XR.OpenXR.Features.HisSupport.HisHandDll;
using static UnityEngine.XR.OpenXR.Features.HisSupport.HisHandPose;

namespace UnityEngine.XR.OpenXR.Features.HisSupport
{
    public enum HandType
    {
        Left,
        Right
    }
    public class HisHandManager : MonoBehaviour
    {
        public static HisHandManager _Instance;
        public static HisHandManager Instance
        {
            get
            {
                if (_Instance == null)
                    _Instance = (HisHandManager)FindObjectOfType(typeof(HisHandManager));

                return _Instance;
            }
        }

        public bool isHandOn
        {
            get
            {
                return mIsHandOn;
            }
            set
            {
                mIsHandOn = value;
            }
        }

        private bool mIsHandOn;
        public int currentHandIndex;

        private bool mIsStarted;

        private HisHand mLeftHand;
        private HisHand mRightHand;
        private Vector3 mHeadPos;
        private Quaternion mHeadQua;

        private IntPtr cpp_Struct_HandInfo;
        private float mHandScale = 1f;
#pragma warning disable 414
        private bool mIsHandTrackStarted = false;
#pragma warning restore
        private WaitForSeconds mWaitForSeconds = new WaitForSeconds(1);
        private string mInfoCase = string.Empty;

        private HxrHiHGInfoExt mHandInfo;

        private void Awake()
        {
            Debug.Log("HisHandManager:Init");
#if UNITY_ANDROID && !UNITY_EDITOR
            HisHandDll.HxrInitializeHandTracking();
#endif
        }

        void Start()
        {
            mLeftHand = this.transform.Find("LeftHand").GetComponent<HisHand>();
            mRightHand = this.transform.Find("RightHand").GetComponent<HisHand>();
        }
        IEnumerator StartHandTracking()
        {
            HisHandDll.HxrHandTrackingStart();
            yield return mWaitForSeconds;
            IntPtr temp;
            temp = HisHandDll.HxrHandTrackingCommunication(0, "");
            while ((Marshal.PtrToStringAnsi(temp)).Length <= 1)
            {
                yield return mWaitForSeconds;
                temp = HisHandDll.HxrHandTrackingCommunication(0, "");
            }
            Debug.Log("HisHandManager:StartHandTrackingSuccess");
            mInfoCase = Marshal.PtrToStringAnsi(temp);
            SendMessageToSo();
            mIsHandTrackStarted = true;
        }
        void LateUpdate()
        {
#if UNITY_ANDROID && !UNITY_EDITOR
        if (!mIsHandOn) return;
        //TODO if (HisControllerManager.controllerLink.mainHandID != 2) return;    

        if (mIsHandTrackStarted)
        {
            HisHandDll.HxrHandTrackingGetData(cpp_Struct_HandInfo);
            mHandInfo = (HxrHiHGInfoExt)Marshal.PtrToStructure(cpp_Struct_HandInfo, typeof(HxrHiHGInfoExt));
            ReceivedHandInfo(mHandInfo);
        }
#elif UNITY_EDITOR
            if (mLeftHand.handJoints.Length <= 0) return;
            mHandInfo.hand_scale = 1;
            mHandInfo.hands = 3;
            HxrXrVector3f leftPos, rightPos;
            leftPos.x = -0.2f;
            leftPos.y = 0f;
            leftPos.z = 0.4f;
            rightPos.x = 0.2f;
            rightPos.y = 0f;
            rightPos.z = 0.4f;
            mHandInfo.left.joints = new HxrXrPosef[1];
            mHandInfo.right.joints = new HxrXrPosef[1];

            mHandInfo.left.joints[0].position = leftPos;
            mHandInfo.right.joints[0].position = rightPos;

            HxrXrQuaternionf leftOri, rightOri;
            Quaternion leftOriQ, rightOriQ;
            leftOriQ = mLeftHand.handJoints[0].rotation;
            leftOri.x = leftOriQ.x;
            leftOri.y = leftOriQ.y;
            leftOri.z = leftOriQ.z;
            leftOri.w = leftOriQ.w;

            rightOriQ = mRightHand.handJoints[0].rotation;
            rightOri.x = rightOriQ.x;
            rightOri.y = rightOriQ.y;
            rightOri.z = rightOriQ.z;
            rightOri.w = rightOriQ.w;
            mHandInfo.left.joints[0].orientation = leftOri;
            mHandInfo.right.joints[0].orientation = rightOri;
            ReceivedHandInfo(mHandInfo);
#endif
        }
        void OnDestroy()
        {
            if (!mIsHandOn) return;
            StopHandTrack();
        }

        public void StartHandTrack()
        {
            if (mIsStarted) return;
            Debug.Log("HisHandManager:StartHandTrack");
            mIsStarted = true;
#if UNITY_ANDROID && !UNITY_EDITOR
                cpp_Struct_HandInfo = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(HxrHiHGInfoExt)));
                Debug.Log("Start Hand");
                StartCoroutine(StartHandTracking());
#endif
        }

        public void StopHandTrack()
        {
            if (!mIsStarted) return;
            Debug.Log("HisHandManager:StopHandTrack");
            StopCoroutine("StartHandTracking");
            mIsStarted = false;
            mLeftHand.gameObject.SetActive(false);
            mRightHand.gameObject.SetActive(false);
#if UNITY_ANDROID && !UNITY_EDITOR
        Marshal.FreeHGlobal(cpp_Struct_HandInfo);
        HisHandDll.HxrHandTrackingStop();
        mIsHandTrackStarted = false;
#endif
        }

        bool isFirstTime = true;
        void OnApplicationPause(bool pause)
        {
            if (!mIsHandOn) return;
            if (!pause)//重新激活
            {
                if (isFirstTime)
                {
                    isFirstTime = false;
                    return;
                }
                SendMessageToSo();
            }
        }

        void ReceivedHandInfo(HxrHiHGInfoExt handInfo)
        {
            mHandScale = handInfo.hand_scale;
            currentHandIndex = handInfo.hands;
            switch (handInfo.hands)
            {
                case 0:
                    if (mLeftHand.gameObject.activeSelf)
                    {
                        mLeftHand.gameObject.SetActive(false);
                    }
                    if (mRightHand.gameObject.activeSelf)
                    {
                        mRightHand.gameObject.SetActive(false);
                    }
                    break;
                case 1:
                    if (mRightHand.gameObject.activeSelf)
                    {
                        mRightHand.gameObject.SetActive(false);
                    }

                    if (!mLeftHand.gameObject.activeSelf)
                        mLeftHand.gameObject.SetActive(true);

                    mLeftHand.UpdateHandScale(mHandScale);
                    mLeftHand.UpdateHandInfo(handInfo.left.joints);
                    mLeftHand.HandTransformRecognition(handInfo.left.trans_rec);
                    mLeftHand.UpdateAimState();
                    mLeftHand.UpdateRayPose();

                    break;
                case 2:
                    if (mLeftHand.gameObject.activeSelf)
                    {
                        mLeftHand.gameObject.SetActive(false);
                    }
                    if (!mRightHand.gameObject.activeSelf)
                    {
                        mRightHand.gameObject.SetActive(true);
                    }
                    mRightHand.UpdateHandScale(mHandScale);
                    mRightHand.UpdateHandInfo(handInfo.right.joints);
                    mRightHand.HandTransformRecognition(handInfo.right.trans_rec);
                    mRightHand.UpdateAimState();
                    mRightHand.UpdateRayPose();
                    break;
                case 3:
                    mLeftHand.gameObject.SetActive(true);
                    mLeftHand.UpdateHandScale(mHandScale);
                    mLeftHand.UpdateHandInfo(handInfo.left.joints);
                    mLeftHand.HandTransformRecognition(handInfo.left.trans_rec);
                    mLeftHand.UpdateAimState();
                    mLeftHand.UpdateRayPose();

                    mRightHand.gameObject.SetActive(true);
                    mRightHand.UpdateHandScale(mHandScale);
                    mRightHand.UpdateHandInfo(handInfo.right.joints);
                    mRightHand.HandTransformRecognition(handInfo.right.trans_rec);
                    mRightHand.UpdateAimState();
                    mRightHand.UpdateRayPose();
                    break;
            }
        }

        public void SendMessageToSo()
        {
            if (!string.IsNullOrEmpty(mInfoCase))
                StartCoroutine(DelayToSend());
        }

        WaitForSeconds waitForSend = new WaitForSeconds(1f);
        IEnumerator DelayToSend()
        {
            yield return waitForSend;//给SLAM的启动时间,防止获取头的位姿不准确
            InputDevice headDevice = InputDevices.GetDeviceAtXRNode(XRNode.Head);
            if (headDevice != null)
            {
                headDevice.TryGetFeatureValue(CommonUsages.devicePosition, out mHeadPos);
                headDevice.TryGetFeatureValue(CommonUsages.deviceRotation, out mHeadQua);
            }

            string sTemp = string.Format("{0},{1},{2},{3},{4},{5},{6},", mHeadPos.x, mHeadPos.y, mHeadPos.z,
              mHeadQua.x, mHeadQua.y, mHeadQua.z, mHeadQua.w);
            Debug.Log("HeadPos:" + mHeadPos);
            Debug.Log("HeadQua:" + mHeadQua);
            HisHandDll.HxrHandTrackingCommunication(3, sTemp);
        }


        #region 对外接口

        /// <summary>
        /// 获取右手激活状态
        /// </summary>
        /// <returns></returns>
        public bool GetRightHandActive()
        {
            return mRightHand.gameObject.activeSelf;
        }

        /// <summary>
        /// 获取右手射线方向
        /// </summary>
        /// <returns></returns>
        public Vector3 GetRightHandRayDir()
        {
            return mRightHand.GetHandRayDir();
        }

        /// <summary>
        /// 获取右手射线原点
        /// </summary>
        /// <returns></returns>
        public Vector3 GetRightHandRayOrigin()
        {
            return mRightHand.GetHandRayOrigin();
        }

        /// <summary>
        /// 获取左手射线方向
        /// </summary>
        /// <returns></returns>
        public Vector3 GetLeftHandRayDir()
        {
            return mLeftHand.GetHandRayDir();
        }

        /// <summary>
        /// 获取左手射线原点
        /// </summary>
        /// <returns></returns>
        public Vector3 GetLeftHandRayOrigin()
        {
            return mLeftHand.GetHandRayOrigin();
        }


        private bool mIsLeftHandClickDown, mIsLeftHandClick, mIsLeftHandClickUp;
        private bool mIsRightHandClickDown, mIsRightHandClick, mIsRightHandClickUp;

        /// <summary>
        /// 获取确认动作触发状态
        /// </summary>
        /// <param name="trackType"></param>
        public bool GetHandClickDown(TrackType trackType)
        {
            switch (trackType)
            {
                case TrackType.Any:
                    return mIsLeftHandClickDown || mIsRightHandClickDown;
                case TrackType.Left:
                    return mIsLeftHandClickDown;
                case TrackType.Right:
                    return mIsRightHandClickDown;
            }
            return false;
        }

        /// <summary>
        /// 设置确认动作触发状态
        /// </summary>
        /// <param name="trackType"></param>手类型<summary>
        public void SetHandClickDown(TrackType trackType, bool state)
        {
            switch (trackType)
            {
                case TrackType.Any:
                    mIsLeftHandClickDown = state;
                    mIsRightHandClickDown = state;
                    break;
                case TrackType.Left:
                    mIsLeftHandClickDown = state;
                    break;
                case TrackType.Right:
                    mIsRightHandClickDown = state;
                    break;
            }
        }
        /// <summary>
        /// 获取确认动作持续触发状态
        /// </summary>
        /// <param name="trackType"></param>手类型<summary>
        public bool GetHandClick(TrackType trackType)
        {
            switch (trackType)
            {
                case TrackType.Any:
                    return mIsLeftHandClick || mIsRightHandClick;
                case TrackType.Left:
                    return mIsLeftHandClick;
                case TrackType.Right:
                    return mIsRightHandClick;
            }
            return false;
        }

        /// <summary>
        ///设置确认动作持续触发状态
        /// </summary>
        /// <param name="trackType">手类型</param>
        public void SetHandClick(TrackType trackType, bool state)
        {
            switch (trackType)
            {
                case TrackType.Any:
                    mIsLeftHandClick = state;
                    mIsRightHandClick = state;
                    break;
                case TrackType.Left:
                    mIsLeftHandClick = state;
                    break;
                case TrackType.Right:
                    mIsRightHandClick = state;
                    break;
            }
        }

        /// <summary>
        /// 获取确认动作取消状态
        /// </summary>
        /// <param name="trackType"></param>
        public bool GetHandClickUp(TrackType trackType)
        {
            switch (trackType)
            {
                case TrackType.Any:
                    return mIsLeftHandClickUp || mIsRightHandClickUp;
                case TrackType.Left:
                    return mIsLeftHandClickUp;
                case TrackType.Right:
                    return mIsRightHandClickUp;
            }
            return false;
        }

        /// <summary>
        /// 设置确认动作取消状态
        /// </summary>
        /// <param name="trackType"></param>
        public void SetHandClickUp(TrackType trackType, bool state)
        {
            switch (trackType)
            {
                case TrackType.Any:
                    mIsLeftHandClickUp = state;
                    mIsRightHandClickUp = state;
                    break;
                case TrackType.Left:
                    mIsLeftHandClickUp = state;
                    break;
                case TrackType.Right:
                    mIsRightHandClickUp = state;
                    break;
            }
        }

        /// <summary>
        /// 配置文件确定动作回调
        /// </summary>
        /// <param name="state"></param>
        public void HandClickDownCallBack(bool state)
        {
            if (state)
            {
                SetHandClickUp(TrackType.Any, false);
            }
            SetHandClickDown(TrackType.Any, state);
        }

        /// <summary>
        /// 配置文件确定动作持续回调
        /// </summary>
        /// <param name="state"></param>
        public void HandClickCallBack(bool state)
        {
            if (state)
            {
                SetHandClickDown(TrackType.Any, false);
            }
            SetHandClick(TrackType.Any, state);
        }

        /// <summary>
        /// 配置文件确定动作取消回调
        /// </summary>
        /// <param name="state"></param>
        public void HandClickUpCallBack(bool state)
        {
            if (state)
            {
                SetHandClickDown(TrackType.Any, false);
                SetHandClick(TrackType.Any, false);
            }
            SetHandClickUp(TrackType.Any, state);
        }

        /// <summary>
        /// 获取手指关节状态
        /// </summary>
        /// <param name="hand"></param>
        /// <param name="jointLocations"></param>
        /// <returns></returns>
        public bool GetJointLocations(HandType hand, ref HandJointLocations jointLocations)
        {
            jointLocations.isActive = 1;
            jointLocations.jointCount = 26;
            jointLocations.handScale = mHandScale;
            jointLocations.jointLocations = new HandJointLocation[26];
            if (hand == HandType.Left)
            {
                for (int i = 0; i < mLeftHand.handJoints.Length; i++)
                {
                    jointLocations.jointLocations[i].pose.pos = mLeftHand.handJoints[i].position;
                    if (i == (int)HandJoint.JointWrist)
                    {
                        jointLocations.jointLocations[i].pose.pos = mLeftHand.handJoints[i].position;
                        jointLocations.jointLocations[i].pose.ori = mLeftHand.handJoints[i].rotation;
                    }
                }
            }
            if (hand == HandType.Right)
            {
                for (int i = 0; i < mRightHand.handJoints.Length; i++)
                {
                    jointLocations.jointLocations[i].pose.pos = mRightHand.handJoints[i].position;
                    if (i == (int)HandJoint.JointWrist)
                    {
                        jointLocations.jointLocations[i].pose.pos = mRightHand.handJoints[i].position;
                        jointLocations.jointLocations[i].pose.ori = mRightHand.handJoints[i].rotation;
                    }
                }
            }
            return true;
        }


        #endregion
    }
}