using UnityEngine;
using UnityEngine.InputSystem;
using UnityEngine.InputSystem.Controls;
using UnityEngine.UI;

namespace Unity.XR.HisenseXR.Samples
{
    public class HisActionBasedKeyEvents : MonoBehaviour
    {
        public Text tipText;

        public InputActionProperty leftTriggerValueActionProperty;
        public InputActionProperty leftTriggerButtonActionProperty;
        public InputActionProperty leftGripValueActionProperty;
        public InputActionProperty leftGripButtonActionProperty;
        public InputActionProperty leftPrimaryButtonActionProperty;
        public InputActionProperty leftSecondaryButtonActionProperty;
        public InputActionProperty leftPrimary2DAxisActionProperty;

        public InputActionProperty rightTriggerValueActionProperty;
        public InputActionProperty rightTriggerButtonActionProperty;
        public InputActionProperty rightGripValueActionProperty;
        public InputActionProperty rightGripButtonActionProperty;
        public InputActionProperty rightPrimaryButtonActionProperty;
        public InputActionProperty rightSecondaryButtonActionProperty;
        public InputActionProperty rightPrimary2DAxisActionProperty;

        InputAction leftTriggerValue;
        InputAction leftTriggerButton;
        InputAction leftGripValue;
        InputAction leftGripButton;
        InputAction leftPrimaryButton;
        InputAction leftSecondaryButton;
        InputAction leftPrimary2DAxis;

        InputAction rightTriggerValue;
        InputAction rightTriggerButton;
        InputAction rightGripValue;
        InputAction rightGripButton;
        InputAction rightPrimaryButton;
        InputAction rightSecondaryButton;
        InputAction rightPrimary2DAxis;

        private void Start()
        {
            leftTriggerValue = leftTriggerValueActionProperty.action;
            leftTriggerButton = leftTriggerButtonActionProperty.action;
            leftGripValue = leftGripValueActionProperty.action;
            leftGripButton = leftGripButtonActionProperty.action;
            leftPrimaryButton = leftPrimaryButtonActionProperty.action;
            leftSecondaryButton = leftSecondaryButtonActionProperty.action;
            leftPrimary2DAxis = leftPrimary2DAxisActionProperty.action;

            rightTriggerValue = rightTriggerValueActionProperty.action;
            rightTriggerButton = rightTriggerButtonActionProperty.action;
            rightGripValue = rightGripValueActionProperty.action;
            rightGripButton = rightGripButtonActionProperty.action;
            rightPrimaryButton = rightPrimaryButtonActionProperty.action;
            rightSecondaryButton = rightSecondaryButtonActionProperty.action;
            rightPrimary2DAxis = rightPrimary2DAxisActionProperty.action;

            rightTriggerButton.performed += m =>
            {
                if (m.ReadValueAsButton())
                {
                    tipText.text = "rightTriggerButton Pressed";
                };
            };
            rightGripButton.performed += m =>
            {
                if (m.ReadValueAsButton())
                {
                    tipText.text = "rightGripButton Pressed";
                }
            };
            rightPrimaryButton.performed += m =>
            {
                if (m.ReadValueAsButton())
                {
                    tipText.text = "rightPrimaryButton Pressed";
                }
            };
            rightSecondaryButton.performed += m =>
            {
                if (m.ReadValueAsButton())
                {
                    tipText.text = "rightSecondaryButton Pressed";
                }
            };

            rightTriggerValue.performed += m =>
            {
                float value = m.ReadValue<float>();
                tipText.text = "rightTriggerValue:" + value;
            };
            rightGripValue.performed += m =>
            {
                float value = m.ReadValue<float>();
                tipText.text = "rightGripValue:" + value;
            };
            rightPrimary2DAxis.performed += m =>
            {
                Vector2 value = m.ReadValue<Vector2>();
                tipText.text = "rightPrimary2DAxis:" + value;
            };

            rightTriggerValue.Enable();
            rightTriggerButton.Enable();
            rightGripValue.Enable();
            rightGripButton.Enable();
            rightPrimaryButton.Enable();
            rightSecondaryButton.Enable();
            rightPrimary2DAxis.Enable();
        }

        private void Update()
        {
            if (CheckButtonAction(leftTriggerButton))
            {
                tipText.text = "leftTriggerButton Pressed";
            }

            if (CheckButtonAction(leftGripButton))
            {
                tipText.text = "leftGripButton Pressed";
            }

            if (CheckButtonAction(leftPrimaryButton))
            {
                tipText.text = "leftPrimaryButton Pressed";
            }

            if (CheckButtonAction(leftSecondaryButton))
            {
                tipText.text = "leftSecondaryButton Pressed";
            }

            if (CheckAxisAction(leftTriggerValue, out float trigger))
            {
                tipText.text = "leftTriggerValue:" + trigger;
            }

            if (CheckAxisAction(leftGripValue, out float grip))
            {
                tipText.text = "leftGripValue:" + grip;
            }

            if (CheckAxis2DAction(leftPrimary2DAxis, out Vector2 primary2D))
            {
                tipText.text = "leftPrimary2DAxis:" + primary2D;
            }
        }

        bool CheckButtonAction(InputAction action)
        {
            if (action == null) return false;

            if (action.activeControl is ButtonControl buttonControl)
                return buttonControl.isPressed;
            return false;
        }

        bool CheckAxisAction(InputAction action, out float result)
        {
            if (action != null)
            {
                if (action.activeControl is AxisControl)
                {
                    result = action.ReadValue<float>();
                    return true;
                }
            }
            result = 0;
            return false;
        }

        bool CheckAxis2DAction(InputAction action, out Vector2 result)
        {
            if (action != null)
            {
                if (action.activeControl is Vector2Control)
                {
                    result = action.ReadValue<Vector2>();
                    return true;
                }
            }
            result = Vector2.zero;
            return false;
        }
    }
}

