using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.XR;

namespace Unity.XR.HisenseXR.Samples
{
    public class HisDeviceBasedKeyEvents : MonoBehaviour
    {
        public XRNode ControllerNode = XRNode.LeftHand;

        public Slider triggerSlider;
        public Slider gripSlider;
        public Image backImage;
        public Image menuImage;
        public Image xButtonImage;
        public Image yButtonImage;
        public Image joystickImage;
        public Image triggerImage;
        public Image gripImage;

        public Text gripText;
        public Text triggerText;

        public Button haptic;

        InputDevice inputDevice;
        RectTransform joyRect;
        float d;

        void Start()
        {
            inputDevice = InputDevices.GetDeviceAtXRNode(ControllerNode);

            List<InputFeatureUsage> featureList = new List<InputFeatureUsage>();
            inputDevice.TryGetFeatureUsages(featureList);

            Debug.Log("InputFeature Usages:");

            foreach (var item in featureList)
            {
                Debug.Log("     :" + item.name);
            }

            joyRect = joystickImage.rectTransform;
            d = joyRect.sizeDelta.x;

            haptic.onClick.AddListener(Haptic);
        }

        void Update()
        {
            if (inputDevice.TryGetFeatureValue(CommonUsages.trigger, out float triggerValue))
            {
                triggerSlider.value = triggerValue;
                triggerText.text = triggerValue.ToString("0.0");
            }

            if (inputDevice.TryGetFeatureValue(CommonUsages.grip, out float gripValue))
            {
                gripSlider.value = gripValue;
                gripText.text = gripValue.ToString("0.0");
            }

            if (inputDevice.TryGetFeatureValue(CommonUsages.primaryButton, out bool primary))
            {
                if (primary)
                {
                    xButtonImage.color = Color.red;
                }
                else
                {
                    xButtonImage.color = Color.white;
                }
            }

            if (inputDevice.TryGetFeatureValue(CommonUsages.secondaryButton, out bool secondary))
            {
                if (secondary)
                {
                    yButtonImage.color = Color.red;
                }
                else
                {
                    yButtonImage.color = Color.white;
                }
            }

            if (inputDevice.TryGetFeatureValue(CommonUsages.menuButton, out bool menu))
            {
                if (menu)
                {
                    menuImage.color = Color.red;
                }
                else
                {
                    menuImage.color = Color.white;
                }
            }

            if (inputDevice.TryGetFeatureValue(CommonUsages.triggerButton, out bool trigger))
            {
                if (trigger)
                {
                    triggerImage.color = Color.red;
                }
                else
                {
                    triggerImage.color = Color.white;
                }
            }

            if (inputDevice.TryGetFeatureValue(CommonUsages.gripButton, out bool grip))
            {
                if (grip)
                {
                    gripImage.color = Color.red;
                }
                else
                {
                    gripImage.color = Color.white;
                }
            }

            if (inputDevice.TryGetFeatureValue(CommonUsages.primary2DAxis, out Vector2 axis))
            {
                joyRect.anchoredPosition = axis * d;
            }
        }

        void Haptic()
        {
            inputDevice.SendHapticImpulse(1, 1);
        }
    }
}